@@ -1,6 +1,6 @@
 package Pcore::App::Controller::Ext;
 
-use Pcore -role, -const;
+use Pcore -role, -const, -l10n;
 use Pcore::Ext;
 use Pcore::Util::Data qw[to_json];
 use Pcore::Util::Scalar qw[is_plain_arrayref];
@@ -11,10 +11,9 @@ with qw[Pcore::App::Controller];
 
 require Pcore::Resources;
 
-has ext_app    => undef;                 # name of the linked application, required
-has ext_title  => 'ExtJS Application';
-has ext_locale => ();
-has ext_theme  => ();
+has ext_app   => undef;                 # name of the linked application, required
+has ext_title => 'ExtJS Application';
+has ext_theme => ();
 
 has _cache => ();
 
@@ -39,17 +38,14 @@ sub build_theme ( $self, $req, $type ) {
     return $theme;
 }
 
-sub build_locale ( $self, $req, $type ) {
-    ( my $locale ) = $req->{env}->{QUERY_STRING} =~ m/locale=([[:alpha:]-]+)/sm;
-
-    return $locale;
-}
-
 around run => sub ( $orig, $self, $req ) {
     if ( $req->{path_tail} ) {
         if ( $req->{path_tail} eq 'app.js' ) {
             $self->_return_app($req);
         }
+        elsif ( $req->{path_tail} eq 'overrides.js' ) {
+            $self->_return_overrides($req);
+        }
         elsif ( $req->{path_tail} eq 'locale.js' ) {
             $self->_return_locale($req);
         }
@@ -82,9 +78,7 @@ sub _return_html ( $self, $req ) {
 
     my $theme = $self->build_theme( $req, $app->{ext_type} ) || $self->{ext_theme} || ( $app->{ext_type} eq 'classic' ? $DEFAULT_THEME_CLASSIC : $DEFAULT_THEME_MODERN );
 
-    my $locale = $self->build_locale( $req, $app->{ext_type} ) || $self->{ext_locale} || 'en';
-
-    if ( !$self->{_cache}->{html}->{$theme}->{$locale} ) {
+    if ( !$self->{_cache}->{html}->{$theme} ) {
         my $resources = [ ( $self->build_resources( $req, $app->{ext_type} ) // [] )->@* ];
 
         # pcore/api.js
@@ -99,10 +93,8 @@ sub _return_html ( $self, $req ) {
             $self->{app}->{devel}
         )->@*;
 
-        # ExtJS locale
-        if ( $locale ne 'en' ) {
-            push $resources->@*, qq[<script src="$self->{path}locale.js?locale=$locale" integrity="" crossorigin="anonymous"></script>];
-        }
+        # overrides
+        push $resources->@*, qq[<script src="$self->{path}overrides.js" integrity="" crossorigin="anonymous"></script>];
 
         # TODO calc checksum 'sha384-' . P->digest->sha384_b64( $res->{body}->$* );
         push $resources->@*, qq[<script src="$self->{path}app.js" integrity="" crossorigin="anonymous"></script>];
@@ -111,7 +103,7 @@ sub _return_html ( $self, $req ) {
         push $resources->@*, Pcore::Resources->fontawesome;
 
         # generate HTML tmpl
-        $self->{_cache}->{html}->{$theme}->{$locale} = \P->text->encode_utf8(
+        $self->{_cache}->{html}->{$theme} = \P->text->encode_utf8(
             P->tmpl->render(
                 'ext/index.html',
                 {   INDEX => {    #
@@ -123,47 +115,66 @@ sub _return_html ( $self, $req ) {
         );
     }
 
-    $req->( 200, [ 'Content-Type' => 'text/html; charset=UTF-8' ], $self->{_cache}->{html}->{$theme}->{$locale} )->finish;
+    $req->( 200, [ 'Content-Type' => 'text/html; charset=UTF-8' ], $self->{_cache}->{html}->{$theme} )->finish;
 
     return;
 }
 
+sub _return_overrides ( $self, $req ) {
+    if ( !$self->{_cache}->{overrides} ) {
+        my $app = $Pcore::Ext::APP->{ $self->{ext_app} };
+
+        my $js = $Pcore::Ext::EXT->{ $app->{ext_type} };
+
+        $self->{_cache}->{overrides}->{js} = $self->_prepare_js($js);
+
+        $self->{_cache}->{overrides}->{etag} = 'W/' . P->digest->md5_hex( $self->{_cache}->{overrides}->{js}->$* );
+    }
+
+    if ( $req->{env}->{HTTP_IF_NONE_MATCH} && $req->{env}->{HTTP_IF_NONE_MATCH} eq $self->{_cache}->{overrides}->{etag} ) {
+        $req->(304)->finish;    # not modified
+    }
+    else {
+        $req->( 200, [ 'Content-Type' => 'application/javascript', 'Cache-Control' => 'must-revalidate', Etag => $self->{_cache}->{overrides}->{etag} ], $self->{_cache}->{overrides}->{js} )->finish;
+    }
+
+    return;
+}
+
+# TODO load only domains, used by app
+# TODO add overrides domains
 sub _return_locale ( $self, $req ) {
+    state $locale_settings = {};
 
     # get locale from query param
     ( my $locale ) = $req->{env}->{QUERY_STRING} =~ m/locale=([[:alpha:]-]+)/sm;
 
-    if ( !$locale ) {
+    if ( !$locale || !$self->{app}->get_locales->{$locale} ) {
         $req->(404)->finish;
 
         return;
     }
 
     if ( !$self->{_cache}->{locale}->{$locale} ) {
-        my $app = $Pcore::Ext::APP->{ $self->{ext_app} };
 
-        my $locale_path = $ENV->{share}->get( 'Pcore-Resources', 'www', "static/ext/$app->{ext_ver}/$app->{ext_type}/locale/locale-${locale}.js" );
-
-        # locale wasn't found
-        if ( !$locale_path ) {
-            $req->(404)->finish;
-
-            return;
-        }
-
-        my $js = P->file->read_text($locale_path)->$*;
+        $locale_settings->{$locale} //= $ENV->{share}->read_cfg( 'data', "ext/locale/$locale.perl" );
 
         # load locale domains
         Pcore::Core::L10N::load_locale($locale) if !exists $Pcore::Core::L10N::MESSAGES->{$locale};
 
         my $plural_form_exp = $Pcore::Core::L10N::LOCALE_PLURAL_FORM->{$locale}->{exp} // 0;
 
-        $js .= <<"JS";
-            Ext.L10N.pluralFormExp = function (n) { return $plural_form_exp; };
-            Ext.L10N.messages = @{[ to_json( $Pcore::Core::L10N::MESSAGES->{$locale}, canonical => 1 )->$* ]};
+        my $js = <<"JS";
+            Ext.L10N.addLocale(
+                '$locale',
+                {   messages: @{[ to_json( $Pcore::Core::L10N::MESSAGES->{$locale}, canonical => 1 )->$* ]},
+                    pluralFormExp: function (n) { return $plural_form_exp; },
+                    settings: @{[ to_json( $locale_settings->{$locale}, canonical => 1 )->$* ]}
+                }
+            );
 JS
 
-        $self->{_cache}->{locale}->{$locale}->{js} = \P->text->encode_utf8( $self->_prepare_js($js)->$* );
+        $self->{_cache}->{locale}->{$locale}->{js} = $self->_prepare_js($js);
 
         $self->{_cache}->{locale}->{$locale}->{etag} = 'W/' . P->digest->md5_hex( $self->{_cache}->{locale}->{$locale}->{js}->$* );
     }
@@ -186,21 +197,23 @@ sub _return_app ( $self, $req ) {
 
         my $data = {
             api_path => $self->{app}->{router}->get_host_api_path( $req->{host} ),
-            api_map  => to_json( {
-                type    => 'websocket',                                                 # remoting
-                url     => $self->{app}->{router}->get_host_api_path( $req->{host} ),
-                actions => $ext_app->{api},
+            api_map  => to_json(
+                {   type    => 'websocket',                                                 # remoting
+                    url     => $self->{app}->{router}->get_host_api_path( $req->{host} ),
+                    actions => $ext_app->{api},
 
-                # not mandatory options
-                # id              => 'api',
-                namespace       => 'EXTDIRECT.' . ref( $self->{app} ) =~ s[::][]smgr,
-                timeout         => 0,                                                   # milliseconds, 0 - no timeout
-                version         => undef,
-                maxRetries      => 0,                                                   # number of times to re-attempt delivery on failure of a call
-                headers         => {},
-                enableBuffer    => 10,                                                  # \1, \0, milliseconds
-                enableUrlEncode => undef,
-            } ),
+                    # not mandatory options
+                    # id              => 'api',
+                    namespace       => 'EXTDIRECT.' . ref( $self->{app} ) =~ s[::][]smgr,
+                    timeout         => 0,                                                   # milliseconds, 0 - no timeout
+                    version         => undef,
+                    maxRetries      => 0,                                                   # number of times to re-attempt delivery on failure of a call
+                    headers         => {},
+                    enableBuffer    => 10,                                                  # \1, \0, milliseconds
+                    enableUrlEncode => undef,
+                },
+                canonical => 1
+            ),
         };
 
         my $js = <<"JS";
@@ -225,7 +238,7 @@ sub _return_app ( $self, $req ) {
                     listenEvents: null,
                     onConnect: function(api) {},
                     onDisconnect: function(api, status, reason) {},
-                    onEvent: function(api, ev) {},
+                    onEvent: function(api, ev) { Ext.fireEvent('remoteEvent', ev) },
                     onListen: function(api, events) {},
                     onRpc: null
                 }),
@@ -233,7 +246,7 @@ sub _return_app ( $self, $req ) {
             });
 JS
 
-        $self->{_cache}->{app}->{js} = \P->text->encode_utf8( $self->_prepare_js($js)->$* );
+        $self->{_cache}->{app}->{js} = $self->_prepare_js($js);
 
         $self->{_cache}->{app}->{etag} = 'W/' . P->digest->md5_hex( $self->{_cache}->{app}->{js}->$* );
     }
@@ -249,6 +262,8 @@ JS
 }
 
 sub _prepare_js ( $self, $js ) {
+    P->text->encode_utf8($js);
+
     if ( $self->{app}->{devel} ) {
         $js = JavaScript::Beautifier::js_beautify(
             $js,
